/******************************************************************************
* 		                                         COPYRIGHT (c) MOTOROLA 2001   
* FILE NAME: pwmcontroller.C                                                      
*                                                                           
* PURPOSE: Port M PWM Controller for Reproducing Audio  											                                                                            
*                                                                           
* DOCUMENTATION SOURCE: EKB Apps                                          
*                                                                           
* TARGET DEVICE: MC9S12DP256                                               
*                                                                                                                                                                                        
* INCLUDE FILES:  basic.h                                                
*                                                                                                                                                      
* AUTHOR: Grant More LOCATION: EKB      LAST EDIT DATE: 14.10.03
*                                                                         
* UPDATE HISTORY                                                         
* REV      AUTHOR       DATE       DESCRIPTION OF CHANGE                  
* ---      ------      ---------    ---------------------                 
* 1.0      G More      02/11/01     initial coding 
* 1.1	   G More	   05/12/01	 	anti-thump functionality added                     
* 1.2	   G More	   14/11/03	    tidy-up and stand-alone build added
*
******************************************************************************/                                                                        

/*===========================================================================*/
/* Motorola reserves the right to make changes without further notice to any */
/* product herein to improve reliability, function, or design. Motorola does */
/* not assume any  liability arising  out  of the  application or use of any */
/* product,  circuit, or software described herein;  neither  does it convey */
/* any license under its patent rights  nor the  rights of others.  Motorola */
/* products are not designed, intended,  or authorized for use as components */
/* in  systems  intended  for  surgical  implant  into  the  body, or  other */
/* applications intended to support life, or  for any  other application  in */
/* which the failure of the Motorola product  could create a situation where */
/* personal injury or death may occur. Should Buyer purchase or use Motorola */
/* products for any such intended  or unauthorized  application, Buyer shall */
/* indemnify and  hold  Motorola  and its officers, employees, subsidiaries, */
/* affiliates,  and distributors harmless against all claims costs, damages, */
/* and expenses, and reasonable  attorney  fees arising  out of, directly or */
/* indirectly,  any claim of personal injury  or death  associated with such */
/* unintended or unauthorized use, even if such claim alleges that  Motorola */
/* was negligent regarding the  design  or manufacture of the part. Motorola */
/* and the Motorola logo* are registered trademarks of Motorola Ltd.         */
/*****************************************************************************/

/* include files */
#include "pwmcontroller.h"

/* global variables */
int sampleCounter=2, 	/*Variable used to count current sample location in array. Offset*/ 
						/*at 2 to avoid counting 1st 2 bytes of size data */
	samplePlayFlag=0,	/*Used to flag main procedure that end of array has been detected */  
	sampleControl=0,	/*Used to control value of current page data is being read from */ 
	z,					/*Simple loop control variable */
	switchPress = 0;	/*Variable used to flag a start condition to main procedure when*/
						/*interrupt is detected*/
unsigned long arraySize;			/*Stores size of current array */
const unsigned char *far wavPtr;	/*Far pointer used to mark current point in paged memory*/

union 	{								/*Union to allow type long variable (array size) to */
       	unsigned long wavSize_long; 	/*be read in 2 chunks and then made a long */ 
       	unsigned char wavSize_char[2];
       	} wavSize;
	
/******************************************************************************
Function Name	:	main
Engineer		:	G.More
Date			:	09.11.01
Parameters		:	NONE
Returns			:	NONE
Notes			:	main routine 
******************************************************************************/
int
main(void) 
{
		/*Initialise Port J */		/*Assumes use of Port J - pin 6 for interrupt*/
	Pim.piej.byte = 0x40;			/*Enable interrupts on port J pin 6*/
	Pim.ppsj.byte = 0x00;			/*Select falling edge polarity for interrupt*/
	Pim.perj.byte = 0x40;			/*Enable pull-up device on port J pin 6*/
	  	
		/*Initialize PWM Module*/
	Pwm.pwme.byte = 0x08;			/*Enable PWM channel 3 */			
	Pwm.pwmpol.byte = 0xFF;			/*Duty cycle starts high */
	Pwm.pwmclk.byte = 0x00;			/*Clock B is the clock source */
	Pwm.pwmprclk.byte = 0x00;		/*Clock B is the bus clock*/
	Pwm.pwmcae.byte = 0x00;			/*Left aligned output mode */
	Pwm.pwmctl.byte = 0x00; 		/*8-bit PWM channels */			
			
		/*Set Control Data */
	Pwm.pwmdty[3].byte = 0x80;		/*Duty cycle initialised at 0x80 */
	Pwm.pwmper[3].byte = 0xFF;		/*PWM period set to maximum*/
	
		/*Initialise timer */
	Timer.tscr1.byte = 0x80;		/*Timer system control register set to enable timer*/
	Timer.mcctl.byte = 0x48;		/*Enables modulus down counter and enables loads of count start value*/
	Timer.mcflg.byte = 0x80;		/*Write 1 to modulus down counter register to clear flag*/

	EnableInterrupts;	/*Allows interrupts*/
			
	{	
		while(1) 	/* Forever loop */ 			
		{	
			Rampcone(0x80);			/*Sets speaker cone to central positon to prevent thump*/
			if (switchPress == 1) 	/*If switch press interrrupt flag high*/
			{					
				switchPress = 0; 	/*Reset interrupt call*/
				for (sampleControl=0; sampleControl<=13; sampleControl++) /*Play all n (in this example 13) pages*/
				
				/*Note that while variable in for loop above should be set to (number of WAV pages)-1, depening on*/
				/*number of pages produces by arraycreator.c. Likewise the sampleControl if statements below should*/
				/*be added to or deleted depending of this value.*/
				
				{											
					samplePlayFlag = 0; /*Reset flag from ISR */
	
					if (sampleControl == 0) 	/*If first time around loop*/
					{
						wavPtr = wav01;				/*Point at first sample*/
						Rampcone(*(wav01 + 2) );	/*Move speaker cone to value of first sample*/
					}
				
					if (sampleControl == 1)		/*If second time around loop*/
						wavPtr = wav02;			/*Point at second sample*/
					if (sampleControl == 2)		/*Etc.*/
						wavPtr = wav03;		
					if (sampleControl == 3)
						wavPtr = wav04;		
					if (sampleControl == 4)
						wavPtr = wav05;		
					if (sampleControl == 5)
						wavPtr = wav06;		
					if (sampleControl == 6)
						wavPtr = wav07;		
					if (sampleControl == 7)
						wavPtr = wav08;		
					if (sampleControl == 8)
						wavPtr = wav09;		
					if (sampleControl == 9)
						wavPtr = wav10;		
					if (sampleControl == 10)
						wavPtr = wav11;		
					if (sampleControl == 11)
						wavPtr = wav12;		
					if (sampleControl == 12)
						wavPtr = wav13;		
					if (sampleControl == 13)
						wavPtr = wav14;		
				  /*if (sampleControl == 14)*/	/*Add this if statement if sample is between 224Kb and 240Kb*/ 
				  /*	wavPtr = wav15;	*/		/*See note above*/
					
					wavSize.wavSize_long = 0; 	/*Reset wavesize to 0*/
		        
		       		for (z=0; z<=1; z++)		/*Loop for 2 count */
		       		{
		       			wavSize.wavSize_char[z] = *(wavPtr + z); 	/*Read array page size from first 2 array elements*/
		       		}
		       		        
   					arraySize = ((wavSize.wavSize_long) >> 16);		/*Bitwise shift to move data to readable location*/
   					 
					Timer.mcctl.byte = 0xCC; 		/*Enable Timer*/	
					Timer.mccnt.word = 0x05AB;		/*Start timer (load count down value)*/
					
					while (samplePlayFlag==0)	/*Delay until sample played*/
					{							/*Wait*/
					}						
				}
			}
			else
			{		/*Do nothing (wait) */
			}	
		}
	}
}
/******************************************************************************
Function Name	:	Delay
Engineer		:	M.Gallop
Date			:	02/06/00
Parameters		:	int delayTime
Returns			:	NONE
Notes			:	simple software delay dependent on CPU clock frequency and
					compile strategy. Used to debounce sample play interrupt 
					switch. 
******************************************************************************/
void 
Delay(int delayTime)					
{
  	int  x =0;									/*outer loop counter */
  	char y =0;									/*inner loop counter */

  	for (x=0; x<delayTime; x++)
  	{	
  		for (y=0; y<100; y++); 
	}
} 

/******************************************************************************
Function Name	:	Rampcone
Engineer		:	G.More
Date			:	04/12/01
Parameters		:	unsigned char sampleValue
Returns			:	NONE
Notes			:	function used to ramp speaker speaker cone to prevent 
					sample thump on activation. sampleValue is the value which
					the procedure will centralise the speaker upon.
******************************************************************************/
void
Rampcone(unsigned char sampleValue)
{			
		while (Pwm.pwmdty[3].byte != sampleValue)	/*While the speaker is not at sampleValue*/
	{
		if (Pwm.pwmdty[3].byte < sampleValue)		/*If the cone is below sampleValue*/
			Pwm.pwmdty[3].byte++;					/*Bring it up to sampleValue*/
		else									/*Otherwise*/
			Pwm.pwmdty[3].byte--;					/*Take it down to sampleValue*/
	}
}
 	
/*****************************************************************************/
/* Start of interrupt service routines  */
/* placed into non-banked memory segment as vectors are only 2 bytes */

#pragma CODE_SEG NON_BANKED	
/******************************************************************************
Function Name	:	_switchpressISR
Engineer		:	G.More
Date			:	9/11/01

Parameters		:	NONE
Returns			:	NONE
Notes			:	Sets flag when sample play command interrupt occurs 
******************************************************************************/
#pragma TRAP_PROC
void
_switchpressISR( void ) 
{
	/*Interrupt occurs when pin 6 of port J is grounded*/

	Delay(1000);			/*Wail 1000*/
 	switchPress = 1;		/*Sets flag for main procedure*/
 	Pim.pifj.byte = 0x40; 	/*Reset interrupt*/
}

/******************************************************************************
Function Name	:	_modulusISR
Engineer		:	G.More	
Date			:	9/11/01

Parameters		:	NONE
Returns			:	NONE
Notes			:	Each access plays one sample. Timing controlled by modulus
					down counter. Uses some of the global variables. 
******************************************************************************/
#pragma TRAP_PROC
void
_modulusISR( void ) 
{
	/*Interrupt occurs when modulus down counter reaches 0*/
	Regs.porte.byte = 0xff;
	if (sampleCounter<(arraySize))					/* If not end of file */
	{	
		Pwm.pwmdty[3].byte = *(wavPtr + sampleCounter); 	/* Load sample to PWM duty register */
		sampleCounter ++; 							/*Increment sampleCounter */
	}	
	else
	{
		Timer.mcctl.byte = 0x4C; 	/*Stop modulus down-counter */
		sampleCounter = 2; 			/*Reset sample counter (offset to avoid reading size of file data) */
		samplePlayFlag = 1; 		/*Set flag to inform main sample has been played */
	}
	Timer.mcflg.byte = 0x80; 		/*Reset interrupt*/
	Regs.porte.byte = 0x00;
}

/******************************************************************************
Function Name	:	_dummyISR
Engineer		:	M.Gallop	
Date			:	25/07/01
Parameters		:	NONE
Returns			:	NONE
Notes			:	Interrupt service routine for unused interrupt vectors. 
******************************************************************************/
#pragma TRAP_PROC
void 
_dummyISR( void )
{
    while( 1 )			/* endless cycle */
	{
	}                                
}

/* End of interrupt service routines  */
/* reset code segment to default */
#pragma CODE_SEG DEFAULT