/*****************************************************************************/
/*                                             COPYRIGHT (c) MOTOROLA 2002   */
/*                                                                           */
/* File Name          :    $RCSfile: s12_register.h,v $						 */
/*                                                                           */
/* Current Revision :      $Revision: 1.0 $                                  */
/*                                                                           */
/* PURPOSE: header file for HCS12 register block					 		 */
/*                                                                           */
/*  ***********************************************************************  */
/*  *  THIS CODE IS ONLY INTENDED AS AN EXAMPLE OF CODE FOR THE           *  */
/*  *  METROWERKS COMPILER AND THE HCS12 EVB AND HAS ONLY BEEN GIVEN A    *  */
/*  *  MIMIMUM LEVEL OF TEST. IT IS PROVIDED 'AS SEEN' WITH NO GUARANTEES *  */
/*  *  AND NO PROMISE OF SUPPORT.                                         *  */
/*  ***********************************************************************  */
/*                                                                           */
/*                                                                           */
/* DESCRIPTION:  standard definitions for HCS12 core registers block         */
/* Defines the external bus ports (A, B & E), system registers and bdm       */
/* registers as a datastructure of type tREGISTER.                           */
/* 																		     */
/* Datastructure base address (out of reset) -                               */
/*                                                                           */
/* HCS12 0x0000 		                                                     */
/*																			 */
/* The datastructure compiled can be determined by the #define's S12DP256,   */
/* S12xxx, etc. These definitions can be found in "s12_common.h".            */
/* The MCU id can be #defined in a "target.h" header file or included as a   */
/* complier command line argument e.g: -DS12DP256.                           */
/* This file provides support for core versions up to v1.5. Core v1.5 support*/
/* can be enabled by defining COREV1_5.										 */
/*																		 	 */
/* Multiple register descriptions are used when a differing bit structure    */
/* exists between parts. In the case of additional bit definitions within    */
/* the same register, then only the full register description is used.       */
/*                                                                           */
/*                                                                           */
/* AUTHOR: r32151/r58711     LOCATION: SPSEKB      LAST EDIT DATE: 13.09.02  */
/*                                                                           */
/* UPDATE HISTORY                                                            */
/* REV  AUTHOR       DATE       DESCRIPTION OF CHANGE                        */
/* ---  ------      --------    ---------------------                        */
/* 1.00	r32151  	28/4/00     - initial coding                             */
/* 1.01	r32151  	09/05/00  	- corrected missing def for partid,          */
/*								- fixed typos in def of itest, mtst0 & mtst1.*/
/*								- Added some bits detail to partid comments. */
/* 1.02 r32151  	10/05/00 	- added missing 't' to tPARTID			     */
/*     							- Changed #include <s12_common.h> 		     */
/* 							      to #include "s12_common.h"			     */
/*  	   						- changed bit/mask names on Ports A, B       */
/*                                and E consistant with PIM ports            */
/* 1.03 r32151      25/05/00 	- corrected typo in typedef of uDDRA1.0      */   
/* 1.04 r32151      25/07/01	- changed CPU id MCSDP256 to S12DP256        */
/*                              - Added disclaimer and reformated file info. */
/* 1.1  r32151      11/03/02	- Updated file info.				         */
/*                              - Modified revision numbering to match RCS   */
/* 1.2  r32151      14/03/02	- made all registers volatile 	 			 */
/* 2.0  r58711      12/09/02    - modified map for v1.5 core (C32) 			 */
/* 2.1  r58711		13/09/02	- made all reserved areas volatile   		 */
/*                                                                           */
/*===========================================================================*/
/* Motorola reserves the right to make changes without further notice to any */
/* product herein to improve reliability, function, or design. Motorola does */
/* not assume any  liability arising  out  of the  application or use of any */
/* product,  circuit, or software described herein;  neither  does it convey */
/* any license under its patent rights  nor the  rights of others.  Motorola */
/* products are not designed, intended,  or authorized for use as components */
/* in  systems  intended  for  surgical  implant  into  the  body, or  other */
/* applications intended to support life, or  for any  other application  in */
/* which the failure of the Motorola product  could create a situation where */
/* personal injury or death may occur. Should Buyer purchase or use Motorola */
/* products for any such intended  or unauthorized  application, Buyer shall */
/* indemnify and  hold  Motorola  and its officers, employees, subsidiaries, */
/* affiliates,  and distributors harmless against all claims costs, damages, */
/* and expenses, and reasonable  attorney  fees arising  out of, directly or */
/* indirectly,  any claim of personal injury  or death  associated with such */
/* unintended or unauthorized use, even if such claim alleges that  Motorola */
/* was negligent regarding the  design  or manufacture of the part. Motorola */
/* and the Motorola logo* are registered trademarks of Motorola Ltd.         */
/*****************************************************************************/

#ifndef S12_REGISTER_H      /*prevent duplicated includes */
#define S12_REGISTER_H

#ifndef S12_COMMON_H        /*prevent duplicated includes */
#include "S12_common.h"
#endif

typedef union uPORTA
  {
  tU08  byte;
  struct
    {
    tU08 pta0    :1;     /*i/o port pins */
    tU08 pta1    :1;
    tU08 pta2    :1;
    tU08 pta3    :1;
    tU08 pta4    :1;
    tU08 pta5    :1;
    tU08 pta6    :1;
    tU08 pta7    :1;
    }bit;
  }tPORTA;

#define PTA0     0x01    /*bit masks */
#define PTA1     0x02
#define PTA2     0x04
#define PTA3     0x08
#define PTA4     0x10
#define PTA5     0x20
#define PTA6     0x40
#define PTA7     0x80


typedef union uPORTB
  {
  tU08  byte;
  struct
    {
    tU08 ptb0    :1;     /*i/o port pins */
    tU08 ptb1    :1;
    tU08 ptb2    :1;
    tU08 ptb3    :1;
    tU08 ptb4    :1;
    tU08 ptb5    :1;
    tU08 ptb6    :1;
    tU08 ptb7    :1;
    }bit;
  }tPORTB;

#define PTB0     0x01    /*bit masks */
#define PTB1     0x02
#define PTB2     0x04
#define PTB3     0x08
#define PTB4     0x10
#define PTB5     0x20
#define PTB6     0x40
#define PTB7     0x80


typedef union uDDRA
  {
  tU08  byte;
  struct
    {
    tU08 ddra0   :1;     /*data direction bits (0:input;1:output) */
    tU08 ddra1   :1;
    tU08 ddra2   :1;
    tU08 ddra3   :1;
    tU08 ddra4   :1;
    tU08 ddra5   :1;
    tU08 ddra6   :1;
    tU08 ddra7   :1;
    }bit;
  }tDDRA;

#define DDRA0    0x01    /*bit masks */
#define DDRA1    0x02
#define DDRA2    0x04
#define DDRA3    0x08
#define DDRA4    0x10
#define DDRA5    0x20
#define DDRA6    0x40
#define DDRA7    0x80


typedef union uDDRB
  {
  tU08  byte;
  struct
    {
    tU08 ddrb0   :1;     /*data direction bits (0:input;1:output) */
    tU08 ddrb1   :1;
    tU08 ddrb2   :1;
    tU08 ddrb3   :1;
    tU08 ddrb4   :1;
    tU08 ddrb5   :1;
    tU08 ddrb6   :1;
    tU08 ddrb7   :1;
    }bit;
  }tDDRB;

#define DDRB0    0x01    /*bit masks */
#define DDRB1    0x02
#define DDRB2    0x04
#define DDRB3    0x08
#define DDRB4    0x10
#define DDRB5    0x20
#define DDRB6    0x40
#define DDRB7    0x80


typedef union uPORTE
  {
  tU08  byte;
  struct
    {
    tU08 pte0    :1;     /*i/o port pins */
    tU08 pte1    :1;
    tU08 pte2    :1;
    tU08 pte3    :1;
    tU08 pte4    :1;
    tU08 pte5    :1;
    tU08 pte6    :1;
    tU08 pte7    :1;
    }bit;
  }tPORTE;

#define PTE0     0x01    /*bit masks */
#define PTE1     0x02
#define PTE2     0x04
#define PTE3     0x08
#define PTE4     0x10
#define PTE5     0x20
#define PTE6     0x40
#define PTE7     0x80


typedef union uDDRE
  {
  tU08  byte;
  struct
    {
    tU08         :2;
    tU08 ddre2   :1;     /*data direction bits (0:input;1:output) */
    tU08 ddre3   :1;
    tU08 ddre4   :1;
    tU08 ddre5   :1;
    tU08 ddre6   :1;
    tU08 ddre7   :1;
    }bit;
  }tDDRE;

#define DDRE2    0x04    /*bit masks */
#define DDRE3    0x08
#define DDRE4    0x10
#define DDRE5    0x20
#define DDRE6    0x40
#define DDRE7    0x80


typedef union uPEAR
  {
  tU08  byte;
  struct
    {
    tU08        :2;     
    tU08 rdwe   :1;     /*read/write enable */
    tU08 lstre  :1;     /*low strobe enable */
    tU08 neclk  :1;     /*no external eclk */
    tU08 pipoe  :1;     /*pipe status signal output enable */
    tU08        :1;     
    tU08 noacce :1;     /*cpu no access output enable */
    }bit;
  }tPEAR;


#define RDWE    0x04    /*bit masks */
#define LSTRE   0x08
#define NECLK   0x10
#define PIPOE   0x20
#define NOACCE  0x80


typedef union uMODE
  { 
  tU08  byte;
  struct
    {
    tU08 eme    :1;     /*emulate port E */
    tU08 emk    :1;     /*emulate port K  */
    tU08        :1;     
    tU08 ivis   :1;     /*internal visibility */
    tU08        :1;     
    tU08 moda   :1;     /*mode select bits */
    tU08 modb   :1;
    tU08 modc   :1;
    }bit;
  }tMODE;

#define EME     0x01    /*bit masks */
#define EMK     0x02
#define IVIS    0x08
#define MODA    0x20
#define MODB    0x40
#define MODC    0x80


typedef union uPUCR
  {
  tU08  byte;
  struct
    {
    tU08 pupae  :1;     /*pull-up port A enable */
    tU08 pupbe  :1;     /*pull-up port B enable */
    tU08        :2;     
    tU08 pupee  :1;     /*pull-up port E enable */
    tU08        :2;     
    tU08 pupke  :1;     /*pull-up port K enable */
    }bit;
  }tPUCR;

#define PUPAE   0x01    /*bit mask */
#define PUPBE   0x02
#define PUPEE   0x10
#define PUPKE   0x40


typedef union uRDRIV
  {
  tU08  byte;
  struct
    {
    tU08 rdpa   :1;     /*reduced drive port A */
    tU08 rdpb   :1;     /*reduced drive port B */
    tU08        :2;     
    tU08 rdpe   :1;     /*reduced drive port E */
    tU08        :2;     
    tU08 rdpk   :1;     /*reduced drive port K */
    }bit;
  }tRDRIV;

#define RDPA    0x01    /*bit mask */
#define RDPB    0x02
#define RDPE    0x10
#define RDPK    0x80


typedef union uEBICTL
  {
  tU08  byte;
  struct
    {
    tU08 estr   :1;     /*e-clock free running or bus control signal */
    tU08        :7;     /*not used */
    }bit;
  }tEBICTL;

#define ESTR    0x01    /*bit mask */


typedef union uINITRM
  {
  tU08  byte;
  struct
    {
    tU08        :1;     /*RAM hi or low alignment within 16K page  */
    tU08        :2;     /*not used */
    tU08 ram    :5;     /*internal ram map position */
    }bit;               
  }tINITRM;

#define RAMHAL  0x01    /*bit masks */
#define RAM11   0x08    
#define RAM12   0x10
#define RAM13   0x20
#define RAM14   0x40
#define RAM15   0x80


typedef union uINITRG
  {
  tU08  byte;
  struct
    {
    tU08        :3;     /*not used */
    tU08 reg    :5;     /*internal register map position */
    }bit;
  }tINITRG;

    
#define REG11   0x08    /*bit masks */
#define REG12   0x10
#define REG13   0x20
#define REG14   0x40
#define REG15   0x80


typedef union uINITEE
  {
  tU08  byte;
  struct
    {
    tU08 eeon   :1;     /*internal eeprom on */
    tU08        :3;     /*not used */
    tU08 ee     :4;     /*internal eeprom map position */
    }bit;
  }tINITEE;

#define EEON    0x01    /*bit masks */
#define EE12    0x10
#define EE13    0x20
#define EE14    0x40
#define EE15    0x80


typedef union uMISC
  {
  tU08  byte;
  struct
    {
    tU08 romon  :1;     /*enable flash */
    tU08 romhm  :1;     /*flash only in second half of map */
    tU08 exstr  :2;     /*external access clock stretch */
    tU08        :4;     /*not used */
    }bit;
  }tMISC;

#define ROMON   0x01    /*bit masks */
#define ROMHM   0x02
#define EXSTR0  0x04
#define EXSTR1  0x08


typedef union uMTST     /*memory test registers  */
  {
  tU08  byte;
  struct
    {
    tU08 bit    :8;     
    }bit;
  }tMTST;


typedef union uITCR
  {
  tU08  byte;
  struct
    {
    tU08 adr0   :1;     /*interrupt test control */
    tU08 adr1   :1;
    tU08 adr2   :1;
    tU08 adr3   :1;
    tU08 wrint  :1;
    tU08        :3;     /*not used */
    }bit;
  }tITCR;

#define ADR0    0x01    /*bit masks */
#define ADR1    0x02
#define ADR2    0x04
#define ADR3    0x08
#define WRINT   0x10
  
  
typedef union uITEST
  {
  tU08  byte;
  struct
    {
    tU08 int0   :1;     /*interrupt test register */
    tU08 int2   :1;
    tU08 int4   :1;
    tU08 int6   :1;
    tU08 int8   :1;
    tU08 inta   :1;
    tU08 intc   :1;
    tU08 inte   :1;
    }bit;
  }tITEST;

#define INT0    0x01    /*bit masks */
#define INT2    0x02
#define INT3    0x04
#define INT6    0x08
#define INT8    0x10
#define INTA    0x20
#define INTC    0x40
#define INTE    0x80

typedef union uVREG
  {
  tU08  byte;
  struct
    {
    tU08 lvif   :1;     /*vreg control register */
    tU08 lvie   :1;
    tU08 lvds   :1;
    tU08    	:5;		/*unused */
    }bit;
  }tVREG;

#define LVIF    0x01    /*bit masks */
#define LVIE    0x02
#define LVDS    0x04


typedef union uPARTID   /* bits 1:3 minor mask revs */
  {                     /* bits 4:7 major mask revs - inc fab x'fer */
  tU16  word;           /* bits 8:11 minor family id */
  struct                /* bits 12:15 major family id */
    {
    tU08 partidh;
    tU08 partidl;
    }byte;
  }tPARTID;


typedef union uMEMSIZ0
  {
  tU08  byte;
  struct
    {
    tU08 ram_sw :3;     
    tU08        :1;
    tU08 eep_sw :2;
    tU08        :1;
    tU08 reg_sw :1;
    }bit;
  }tMEMSIZ0;

#define RAM_SW0 0x01    /*bit masks */
#define RAM_SW1 0x02
#define RAM_SW2 0x04
#define EEP_SW0 0x10
#define EEP_SW1 0x20
#define REG_SW0 0x80


typedef union uMEMSIZ1
  {
  tU08  byte;
  struct
    {
    tU08 pag_sw :2;     
    tU08        :4;
    tU08 rom_sw :2;
    }bit;
  }tMEMSIZ1;

#define PAG_SW0 0x01    /*bit masks */
#define PAG_SW1 0x02
#define ROM_SW0 0x40
#define ROM_SW1 0x80


typedef union uINTCR
  {
  tU08  byte;
  struct
    {
    tU08        :6;     /*not used */
    tU08 irqen  :1;     /*external irq enable */
    tU08 irqe   :1;     /*irq select edge sensitive only */
    }bit;
  }tINTCR;

#define IRQEN   0x40    /*bit masks */
#define IRQE    0x80


typedef union uHPRIO
  {
  tU08  byte;
  struct
    {
    tU08        :1;     /*not used */
    tU08 psel1  :1;     /*highest priority interrupt select bits */
    tU08 psel2  :1;
    tU08 psel3  :1;
    tU08 psel4  :1;
    tU08 psel5  :1;
    tU08 psel6  :1;     
    tU08 psel7  :1;     
    }bit;
  }tHPRIO;

#define PSEL1   0x02    /*bit masks */
#define PSEL2   0x04
#define PSEL3   0x08
#define PSEL4   0x10
#define PSEL5   0x20
#define PSEL6   0x40
#define PSEL7   0x80


typedef union uBKPCT0
  {
  tU08  byte;
  struct
    {
    tU08        :4;     /*not used */
    tU08 bktag  :1;     
    tU08 bkbdm  :1;
    tU08 bkfull :1;     
    tU08 bken   :1;     /*breakpoint mode enable */
    }bit;
  }tBKPCT0;

#define BKTAG   0x10
#define BKBDM   0x20
#define BKFULL  0x40
#define BKEN    0x80


typedef union uBKPCT1
  {
  tU08  byte;
  struct
    {
    tU08 bk1rw  :1;    
    tU08 bk1rwe :1;    
    tU08 bk0rw  :1;     
    tU08 bk0rwe :1;   
    tU08 bk1mbl :1;    
    tU08 bk1mbh :1;    
    tU08 bk0mbl :1;     
    tU08 bk0mbh :1;    
    }bit;
  }tBKPCT1;

#define BK1RW   0x01    /*bit masks */
#define BK1RWE  0x02
#define BK0RW   0x04
#define BK0RWE  0x08
#define BK1MBL  0x10
#define BK1MBH  0x20
#define BK0Mbl  0x40
#define BK0MBH  0x80


typedef union uBKP0X
  {
  tU08  byte;
  struct
    {
    tU08 bk0v   :6;     /* */
    tU08        :2;     /*not used */

    }bit;
  }tBKP0X;

#define BK0V0   0x01    /*bit masks */
#define BK0V1   0x02
#define BK0V2   0x04
#define BK0V3   0x08
#define BK0V4   0x10
#define BK0V5   0x20


typedef union uBKP0
  {
  tU16  word;
  struct
    {
    tU08 high;
    tU08 low;
    }byte;
  }tBKP0;


typedef union uBKP1X
  {
  tU08  byte;
  struct
    {
    tU08 bk1v   :6;     /* */
    tU08        :2;     /*not used */

    }bit;
  }tBKP1X;

#define BK1V0   0x01    /*bit masks */
#define BK1V1   0x02
#define BK1V2   0x04
#define BK1V3   0x08
#define BK1V4   0x10
#define BK1V5   0x20


typedef union uBKP1
  {
  tU16  word;
  struct
    {
    tU08 high;
    tU08 low;
    }byte;
  }tBKP1;


typedef union uDBGC1	/*debug control register 1 */
  {
  tU08  byte;
  struct
    {
    tU08 capmod :2;
    tU08        :1;  	/*not used */
    tU08 dbgbrk :1;
    tU08 begin	:1;
    tU08 trgsel	:1;
    tU08 arm   	:1;
    tU08 dbgen 	:1;         
    }bit;
  }tDBGC1;

#define CAPMOD0 0x01	/*bit masks */
#define CAPMOD1 0x02
#define DBGBRK  0x08
#define BEGIN   0x10
#define TRGSEL  0x20
#define ARM		0x40 
#define DBGEN	0x80	
  

typedef union uDBGSC 		/*debug status and control register */
  {
  tU08  byte;
  struct
    {
    tU08 trg    :4;
    tU08  		:1;
    tU08 cf	 	:1;     
    tU08 bf		:1;     
    tU08 af		:1;        
    }bit;
  }tDBGSC;

#define TRG0   	0x01		 /*bit masks */
#define TRG1  	0x02   
#define TRG2 	0x04
#define TRG3  	0x08
#define CF   	0x20
#define BF 		0x40
#define AF		0x80


typedef union uDBGTBH		/*debug trace buffer high register */
  {
  tU08  byte;
  struct
    {
    tU08 dbgtbh8  :1;     
    tU08 dbgtbh9  :1;     
    tU08 dbgtbh10 :1;
    tU08 dbgtbh11 :1;
    tU08 dbgtbh12 :1;
    tU08 dbgtbh13 :1;
    tU08 dbgtbh14 :1;     
    tU08 dbgtbh15 :1;     
    }bit;
  }tDBGTBH;

#define DBGTBH8   	0x01	/*bit masks */
#define DBGTBH9    	0x02   
#define DBGTBH10    0x04
#define DBGTBH11	0x08
#define DBGTBH12   	0x10
#define DBGTBH13   	0x20
#define DBGTBH14   	0x40
#define DBGTBH15   	0x80

typedef union uDBGTBL		/*debug trace buffer low register */
  {
  tU08  byte;
  struct
    {
    tU08 dbgtbl0 :1;     
    tU08 dbgtbl1 :1;     
    tU08 dbgtbl2 :1;
    tU08 dbgtbl3 :1;
    tU08 dbgtbl4 :1;
    tU08 dbgtbl5 :1;
    tU08 dbgtbl6 :1;     
    tU08 dbgtbl7 :1;     
    }bit;
  }tDBGTBL;

#define  DBGTBL0  0x01		 /*bit masks */
#define  DBGTBL1  0x02   
#define  DBGTBL2  0x04
#define  DBGTBL3  0x08
#define  DBGTBL4  0x10
#define  DBGTBL5  0x20
#define  DBGTBL6  0x40
#define  DBGTBL7  0x80


typedef union uDBGTB   		/*combined trace buffer structure */
  {                    
  tU16  word;           
  struct             
    {
    tU08 dbgtbh;
    tU08 dbgtbl;
    }byte;
  }tDBGTB;


typedef union uDBGCNT		/*debug count register */
  {
  tU08  byte;
  struct
    {
	tU08 cnt :6;   
    tU08     :1;			/*unused */     
    tU08 tbf :1;     
    }bit;
  }tDBGCNT;

#define DBCNT0   0x01		/*bit masks */
#define DBCNT1   0x02   
#define DBCNT2   0x04
#define DBCNT3   0x08
#define DBCNT4   0x10
#define DBCNT5   0x20
#define TBF   	 0x80


typedef union uDBGCCX		/*debug comparator C extended register */
  {
  tU08  byte;
  struct
    {
    tU08 extcmp :6;     
    tU08 pagsel :2;         
    }bit;
  }tDBGCCX;

#define EXTCMP0   0x01		 /*bit masks */
#define EXTCMP1   0x02   
#define EXTCMP2   0x04
#define EXTCMP3   0x08
#define EXTCMP4   0x10
#define EXTCMP5   0x20
#define PAGSEL0   0x40
#define PAGSEL1   0x80


typedef union uDBGCCH		/* debug comparator C high register */
  {
  tU08  byte;
  struct
    {
    tU08 dbgcch8  :1;     
    tU08 dbgcch9  :1;     
    tU08 dbgcch10 :1;
    tU08 dbgcch11 :1;
    tU08 dbgcch12 :1;
    tU08 dbgcch13 :1;
    tU08 dbgcch14 :1;     
    tU08 dbgcch15 :1;     
    }bit;
  }tDBGCCH;

#define  DBGCCH8  0x01	 	/*bit masks */
#define  DBGCCH9  0x02   
#define  DBGCCH10 0x04
#define  DBGCCH11 0x08
#define  DBGCCH12 0x10
#define  DBGCCH13 0x20
#define  DBGCCH14 0x40
#define  DBGCCH15 0x80


typedef union uDBGCCL		/* debug comparator C low register */
  {
  tU08  byte;
  struct
    {
    tU08 dbgccl0 :1;     
    tU08 dbgccl1 :1;     
    tU08 dbgccl2 :1;
    tU08 dbgccl3 :1;
    tU08 dbgccl4 :1;
    tU08 dbgccl5 :1;
    tU08 dbgccl6 :1;     
    tU08 dbgccl7 :1;     
    }bit;
  }tDBGCCL;

#define  DBGCCL0  0x01		 /*bit masks */
#define  DBGCCL1  0x02   
#define  DBGCCL2  0x04
#define  DBGCCL3  0x08
#define  DBGCCL4  0x10
#define  DBGCCL5  0x20
#define  DBGCCL6  0x40
#define  DBGCCL7  0x80


typedef union uDBGCC   		/*combined comparator C structure */
  {                    
  tU16  word;           
  struct             
    {
    tU08 dbgcch;
    tU08 dbgccl;
    }byte;
  }tDBGCC;


typedef union uDBGC2		/*debug control register 2 */
  {
  tU08  byte;
  struct
    {
    tU08 rwc	:1;     
    tU08 rwcen	:1;     
    tU08 tagc	:1;
    tU08 bkcen	:1;
    tU08 tagab	:1;
    tU08 bdm	:1;
    tU08 full	:1;     
    tU08 bkaben	:1;     
    }bit;
  }tDBGC2;

#define  RWC	0x01		 /*bit masks */
#define  RWCEN	0x02   
#define  TAGC	0x04
#define  BKCEN  0x08
#define  TAGAB	0x10
#define  BDM	0x20
#define  FULL	0x40
#define  BKABEN 0x80


typedef union uDBGC3		/*debug control register 3 */
  {
  tU08  byte;
  struct
    {
    tU08 rwb	:1;     
    tU08 rwben	:1;     
    tU08 rwa 	:1;
    tU08 rwaen	:1;
    tU08 bkbmbl	:1;
    tU08 bkbmbh :1;
    tU08 bkambl	:1;     
    tU08 bkambh	:1;     
    }bit;
  }tDBGC3;

#define  RWB	0x01		 /*bit masks */
#define  RWBEN	0x02   
#define  RWA	0x04
#define  RWAEN  0x08
#define  BKBMBL	0x10
#define  BKBMBH	0x20
#define  BKAMBL	0x40
#define  BKAMBH 0x80  
  
  
typedef union uDBGCAX		/*debug comparator A extended register */
  {
  tU08  byte;
  struct
    {
    tU08 extcmp :6;     
    tU08 pagsel :2;         
    }bit;
  }tDBGCAX;


typedef union uDBGCAH		/* debug comparator A high register */
  {
  tU08  byte;
  struct
    {
    tU08 dbgcah8  :1;     
    tU08 dbgcah9  :1;     
    tU08 dbgcah10 :1;
    tU08 dbgcah11 :1;
    tU08 dbgcah12 :1;
    tU08 dbgcah13 :1;
    tU08 dbgcah14 :1;     
    tU08 dbgcah15 :1;     
    }bit;
  }tDBGCAH;

#define  DBGCAH8  0x01	 	/*bit masks */
#define  DBGCAH9  0x02   
#define  DBGCAH10 0x04
#define  DBGCAH11 0x08
#define  DBGCAH12 0x10
#define  DBGCAH13 0x20
#define  DBGCAH14 0x40
#define  DBGCAH15 0x80


typedef union uDBGCAL		/* debug comparator A low register */
  {
  tU08  byte;
  struct
    {
    tU08 dbgcal0 :1;     
    tU08 dbgcal1 :1;     
    tU08 dbgcal2 :1;
    tU08 dbgcal3 :1;
    tU08 dbgcal4 :1;
    tU08 dbgcal5 :1;
    tU08 dbgcal6 :1;     
    tU08 dbgcal7 :1;     
    }bit;
  }tDBGCAL;

#define  DBGCAL0  0x01		 /*bit masks */
#define  DBGCAL1  0x02   
#define  DBGCAL2  0x04
#define  DBGCAL3  0x08
#define  DBGCAL4  0x10
#define  DBGCAL5  0x20
#define  DBGCAL6  0x40
#define  DBGCAL7  0x80

typedef union uDBGCA   		/*combined comparator A structure */
  {                    
  tU16  word;           
  struct             
    {
    tU08 dbgcah;
    tU08 dbgcal;
    }byte;
  }tDBGCA;

typedef union uDBGCBX		/*debug comparator B extended register */
  {
  tU08  byte;
  struct
    {
    tU08 extcmp :6;     
    tU08 pagsel :2;         
    }bit;
  }tDBGCBX;


typedef union uDBGCBH		/* debug comparator B high register */
  {
  tU08  byte;
  struct
    {
    tU08 dbgcbh8  :1;     
    tU08 dbgcbh9  :1;     
    tU08 dbgcbh10 :1;
    tU08 dbgcbh11 :1;
    tU08 dbgcbh12 :1;
    tU08 dbgcbh13 :1;
    tU08 dbgcbh14 :1;     
    tU08 dbgcbh15 :1;     
    }bit;
  }tDBGCBH;

#define  DBGCBH8  0x01	 	/*bit masks */
#define  DBGCBH9  0x02   
#define  DBGCBH10 0x04
#define  DBGCBH11 0x08
#define  DBGCBH12 0x10
#define  DBGCBH13 0x20
#define  DBGCBH14 0x40
#define  DBGCBH15 0x80


typedef union uDBGCBL		/* debug comparator B low register */
  {
  tU08  byte;
  struct
    {
    tU08 dbgcbl0 :1;     
    tU08 dbgcbl1 :1;     
    tU08 dbgcbl2 :1;
    tU08 dbgcbl3 :1;
    tU08 dbgcbl4 :1;
    tU08 dbgcbl5 :1;
    tU08 dbgcbl6 :1;     
    tU08 dbgcbl7 :1;     
    }bit;
  }tDBGCBL;

#define  DBGCBL0  0x01		 /*bit masks */
#define  DBGCBL1  0x02   
#define  DBGCBL2  0x04
#define  DBGCBL3  0x08
#define  DBGCBL4  0x10
#define  DBGCBL5  0x20
#define  DBGCBL6  0x40
#define  DBGCBL7  0x80

typedef union uDBGCB   		/*combined comparator B structure */
  {                    
  tU16  word;           
  struct             
    {
    tU08 dbgcbh;
    tU08 dbgcbl;
    }byte;
  }tDBGCB;
  
typedef struct              	/*port and internal resource control */
  {
  volatile tPORTA   porta;  	/*port A data register */
  volatile tPORTB   portb;  	/*port B data register */
  volatile tDDRA    ddra;   	/*port A data direction register */
  volatile tDDRB    ddrb;   	/*port B data direction register */
  volatile tU08     rsv1[4];	/*reserved (maintaining memory map) */
  volatile tPORTE   porte;  	/*port E data register */
  volatile tDDRE    ddre;   	/*port E data direction register */
  volatile tPEAR    pear;   	/*port E assignment register */
  volatile tMODE    mode;   	/*mode register */
  volatile tPUCR    pucr;   	/*pull-up control register */
  volatile tRDRIV   rdriv;  	/*reduced drive register */
  volatile tEBICTL  ebictl; 	/*external bus control */
  volatile tU08     rsv2;   	/*reserved (maintaining memory map) */
  volatile tINITRM  initrm; 	/*initialise internal ram position */
  volatile tINITRG  initrg; 	/*initialise internal register position */
  volatile tINITEE  initee; 	/*initialise internal eeprom position */
  volatile tMISC    misc;   	/*miscellaneous system control register */
  volatile tMTST    mtst0;  	/*memory test (?) */
  volatile tITCR    itcr;   	/*interrupt test control (?) */
  volatile tITEST   itest;  	/*interrupt test status (?) */
  volatile tMTST    mtst1;  	/*memory test (?) */

 
  #ifndef S12C32				/*HCS12 devices with breakpoint module */
 
  volatile tU08     rsv3[2]; 	/*reserved (maintaining memory map) */
  volatile tPARTID  partid;  	/*WORD - part id information */
  volatile tMEMSIZ0 memsiz0; 	/*memory size info 0 */
  volatile tMEMSIZ1 memsiz1; 	/*memory size info 1  */
  volatile tINTCR   intcr;  	/*interrupt control register */
  volatile tHPRIO   hprio;  	/*highest priority I interrupt */
  volatile tU08     rsv4[8];	/*reserved (maintaining memory map) */
  volatile tBKPCT0  bkpct0; 	/*breakpoint control register 0 */
  volatile tBKPCT1  bkpct1; 	/*breakpoint control register 1 */
  volatile tBKP0X   bkp0x;  	/*breakpoint address 0 expansion register */
  volatile tBKP0    bkp0;   	/*WORD - breakpoint address register(s) */
  volatile tBKP1X   bkp1x;  	/*breakpoint address 1 expansion register */
  volatile tBKP1    bkp1;   	/*WORD - breakpoint data register(s) */
 
  #else 						/*HCS12 devices with debug module (C32) */
 
  volatile tU08     rsv3; 		/*reserved (maintaining memory map) */
  volatile tVREG    vreg;	    /*voltage regulator */
  volatile tPARTID  partid;  	/*WORD - part id information */
  volatile tMEMSIZ0 memsiz0; 	/*memory size info 0 */
  volatile tMEMSIZ1 memsiz1; 	/*memory size info 1  */
  volatile tINTCR   intcr;  	/*interrupt control register */
  volatile tHPRIO   hprio;  	/*highest priority I interrupt */ 
  volatile tDBGC1   dbgc1;		/*debug control register 1 */
  volatile tDBGSC   dbgsc;		/*debug status and control register */
  volatile tDBGTB 	dbgtb;		/*WORD - debug trace buffer register */
  volatile tDBGCNT  dbgcnt;		/*debug count register */
  volatile tDBGCCX  dbgccx;		/*debug comparator C extended register */
  volatile tDBGCC   dbgcc;		/*WORD - debug comparator C register(s) */
  volatile tDBGC2 	dbgc2;		/*debug control register 2 */
  volatile tDBGC3   dbgc3;		/*debug control register 3 */
  volatile tDBGCAX  dbgcax;		/*debug comparator A extended register */
  volatile tDBGCA   dbgca;		/*WORD - debug comparator A register(s) */		
  volatile tDBGCBX  dbgcbx;		/*debug comparator B extended register */
  volatile tDBGCB   dbgcb; 		/*WORD - debug comparator C register(s) */
 
  #endif	/*S12C32 */
  }tREGISTER;

#endif /*S12_REGISTER_H */